/**
 * OpenAI GPT Provider Implementation
 * 
 * This module provides integration with OpenAI's GPT models through their official SDK.
 * OpenAI offers industry-leading language models with excellent performance across
 * diverse tasks including reasoning, coding, creative writing, and function calling.
 * 
 * Key Features:
 * - Support for all GPT model variants (GPT-4, GPT-4o, GPT-3.5 Turbo)
 * - Advanced streaming support with real-time token delivery
 * - Native function calling and JSON mode capabilities
 * - Vision support for image understanding (model-dependent)
 * - Organization and project-level access control
 * 
 * OpenAI-Specific Considerations:
 * - Uses "prompt_tokens" and "completion_tokens" terminology
 * - Supports system messages natively within conversation flow
 * - Has sophisticated rate limiting and pricing tiers
 * - Provides extensive model configuration options
 * - Supports batch processing and fine-tuning capabilities
 * 
 * @author Jan-Marlon Leibl
 * @version 1.0.0
 * @see https://platform.openai.com/docs/api-reference
 */

import OpenAI from 'openai';
import type {
  AIProviderConfig,
  CompletionParams,
  CompletionResponse,
  CompletionChunk,
  ProviderInfo,
  AIMessage
} from '../types/index.js';
import { BaseAIProvider } from './base.js';
import { AIProviderError, AIErrorType } from '../types/index.js';

// ============================================================================
// TYPES AND INTERFACES
// ============================================================================

/**
 * Configuration interface for OpenAI provider with OpenAI-specific options.
 * 
 * @example
 * ```typescript
 * const config: OpenAIConfig = {
 *   apiKey: process.env.OPENAI_API_KEY!,
 *   defaultModel: 'gpt-4o',
 *   organization: 'org-your-org-id',
 *   project: 'proj-your-project-id',
 *   timeout: 60000,
 *   maxRetries: 3
 * };
 * ```
 */
export interface OpenAIConfig extends AIProviderConfig {
  /** 
   * Default OpenAI model to use for requests.
   * 
   * Recommended models:
   * - 'gpt-4o': Latest multimodal model, best overall performance
   * - 'gpt-4o-mini': Faster and cheaper, good for simple tasks
   * - 'gpt-4-turbo': Previous generation flagship, excellent reasoning
   * - 'gpt-3.5-turbo': Most cost-effective, good for basic tasks
   * 
   * @default 'gpt-4o'
   */
  defaultModel?: string;

  /** 
   * OpenAI organization ID for billing and access control.
   * 
   * This is useful for organizations with multiple teams or projects
   * that need separate billing and usage tracking.
   * 
   * @see https://platform.openai.com/account/org-settings
   */
  organization?: string;

  /** 
   * OpenAI project ID for granular access control.
   * 
   * Projects allow fine-grained permission management within
   * an organization, enabling better governance and security.
   * 
   * @see https://platform.openai.com/docs/guides/production-best-practices
   */
  project?: string;
}

/**
 * Internal interface for enhanced streaming chunk tracking.
 * OpenAI streaming provides detailed metadata about completion progress.
 */
interface StreamingState {
  /** Current message ID from OpenAI */
  messageId: string;
  /** Accumulated prompt tokens (estimated) */
  promptTokens: number;
  /** Accumulated completion tokens (estimated) */
  completionTokens: number;
  /** Whether the stream has started */
  started: boolean;
}

// ============================================================================
// OPENAI PROVIDER IMPLEMENTATION
// ============================================================================

/**
 * OpenAI GPT provider implementation.
 * 
 * This class handles all interactions with OpenAI's chat completion API through
 * their official TypeScript SDK. It provides optimized handling of OpenAI's
 * features including function calling, vision, and advanced streaming.
 * 
 * Usage Pattern:
 * 1. Create instance with configuration including API key
 * 2. Call initialize() to set up client and validate credentials
 * 3. Use complete() or stream() for text generation
 * 4. Handle any AIProviderError exceptions appropriately
 * 
 * @example
 * ```typescript
 * const openai = new OpenAIProvider({
 *   apiKey: process.env.OPENAI_API_KEY!,
 *   defaultModel: 'gpt-4o',
 *   organization: 'org-your-org-id'
 * });
 * 
 * await openai.initialize();
 * 
 * const response = await openai.complete({
 *   messages: [
 *     { role: 'system', content: 'You are a helpful assistant.' },
 *     { role: 'user', content: 'Explain machine learning.' }
 *   ],
 *   maxTokens: 1000,
 *   temperature: 0.7
 * });
 * ```
 */
export class OpenAIProvider extends BaseAIProvider {
  // ========================================================================
  // INSTANCE PROPERTIES
  // ========================================================================

  /** OpenAI SDK client instance (initialized during doInitialize) */
  private client: OpenAI | null = null;
  
  /** Default model identifier for requests */
  private readonly defaultModel: string;
  
  /** Organization ID for billing and access control */
  private readonly organization?: string;
  
  /** Project ID for granular permissions */
  private readonly project?: string;

  // ========================================================================
  // CONSTRUCTOR
  // ========================================================================

  /**
   * Creates a new OpenAI provider instance.
   * 
   * @param config - OpenAI-specific configuration options
   * @throws {AIProviderError} If configuration validation fails
   */
  constructor(config: OpenAIConfig) {
    super(config);
    
    // Set OpenAI-specific defaults
    this.defaultModel = config.defaultModel || 'gpt-4o';
    this.organization = config.organization;
    this.project = config.project;
    
    // Validate model name format
    this.validateModelName(this.defaultModel);
  }

  // ========================================================================
  // PROTECTED TEMPLATE METHOD IMPLEMENTATIONS
  // ========================================================================

  /**
   * Initializes the OpenAI provider by setting up the client.
   * 
   * This method:
   * 1. Creates the OpenAI SDK client with configuration
   * 2. Tests the connection with a minimal API call
   * 3. Validates API key permissions and model access
   * 4. Sets up organization and project context if provided
   * 
   * @protected
   * @throws {Error} If client creation or connection validation fails
   */
  protected async doInitialize(): Promise<void> {
    try {
      // Create OpenAI client with optimized configuration
      this.client = new OpenAI({
        apiKey: this.config.apiKey,
        baseURL: this.config.baseUrl,
        timeout: this.config.timeout,
        maxRetries: this.config.maxRetries,
        organization: this.organization,
        project: this.project,
        // Enable automatic retries with exponential backoff
        defaultQuery: undefined,
        defaultHeaders: {
          'User-Agent': 'simple-ai-provider/1.0.0'
        }
      });

      // Validate connection and permissions
      await this.validateConnection();
      
    } catch (error) {
      // Clean up on failure
      this.client = null;
      
      throw new AIProviderError(
        `Failed to initialize OpenAI provider: ${(error as Error).message}`,
        AIErrorType.AUTHENTICATION,
        undefined,
        error as Error
      );
    }
  }

  /**
   * Generates a text completion using OpenAI's chat completions API.
   * 
   * This method:
   * 1. Converts messages to OpenAI's format
   * 2. Builds optimized request parameters
   * 3. Makes API call with comprehensive error handling
   * 4. Formats response to standard interface
   * 
   * @protected
   * @param params - Validated completion parameters
   * @returns Promise resolving to formatted completion response
   * @throws {Error} If API request fails
   */
  protected async doComplete(params: CompletionParams): Promise<CompletionResponse> {
    if (!this.client) {
      throw new AIProviderError('OpenAI client not initialized', AIErrorType.INVALID_REQUEST);
    }

    try {
      // Build optimized request parameters
      const requestParams = this.buildRequestParams(params, false);
      
      // Make API request - explicitly type as ChatCompletion for non-streaming
      const response = await this.client.chat.completions.create(requestParams) as OpenAI.Chat.Completions.ChatCompletion;

      // Format and return response
      return this.formatCompletionResponse(response);
      
    } catch (error) {
      throw this.handleOpenAIError(error as Error);
    }
  }

  /**
   * Generates a streaming text completion using OpenAI's streaming API.
   * 
   * This method:
   * 1. Builds streaming request parameters
   * 2. Handles real-time stream chunks from OpenAI
   * 3. Tracks token usage throughout the stream
   * 4. Yields formatted chunks with proper completion tracking
   * 
   * @protected
   * @param params - Validated completion parameters
   * @returns AsyncIterable yielding completion chunks
   * @throws {Error} If streaming request fails
   */
  protected async *doStream(params: CompletionParams): AsyncIterable<CompletionChunk> {
    if (!this.client) {
      throw new AIProviderError('OpenAI client not initialized', AIErrorType.INVALID_REQUEST);
    }

    try {
      // Build streaming request parameters
      const requestParams = this.buildRequestParams(params, true);
      
      // Create streaming request
      const stream = this.client.chat.completions.create(requestParams);

      // Process stream chunks
      yield* this.processStreamChunks(stream);
      
    } catch (error) {
      throw this.handleOpenAIError(error as Error);
    }
  }

  // ========================================================================
  // PUBLIC INTERFACE METHODS
  // ========================================================================

  /**
   * Returns comprehensive information about the OpenAI provider.
   * 
   * @returns Provider information including models, capabilities, and limits
   */
  public getInfo(): ProviderInfo {
    return {
      name: 'OpenAI',
      version: '1.0.0',
      models: [
        'gpt-4o',                    // Latest multimodal model
        'gpt-4o-mini',               // Faster, cheaper variant
        'gpt-4-turbo',               // Previous flagship model
        'gpt-4',                     // Original GPT-4
        'gpt-3.5-turbo',            // Most cost-effective
        'gpt-3.5-turbo-0125',       // Specific version
        'gpt-4o-2024-11-20',        // Latest dated version
        'gpt-4-turbo-2024-04-09'    // Previous dated version
      ],
      maxContextLength: 128000, // 128K tokens for latest models
      supportsStreaming: true,
      capabilities: {
        vision: true,               // GPT-4o supports image understanding
        functionCalling: true,      // Advanced function/tool calling
        jsonMode: true,             // Structured JSON output
        systemMessages: true,       // Native system message support
        reasoning: true,            // Strong reasoning capabilities
        codeGeneration: true,       // Excellent at programming tasks
        multimodal: true,           // Text + image understanding
        fineTuning: true           // Custom model training
      }
    };
  }

  // ========================================================================
  // PRIVATE UTILITY METHODS
  // ========================================================================

  /**
   * Validates the connection by making a minimal test request.
   * 
   * @private
   * @throws {AIProviderError} If connection validation fails
   */
  private async validateConnection(): Promise<void> {
    if (!this.client) {
      throw new Error('Client not initialized');
    }

    try {
      // Make minimal request to test connection and permissions
      await this.client.chat.completions.create({
        model: this.defaultModel,
        messages: [{ role: 'user', content: 'Hi' }],
        max_tokens: 1
      });
      
    } catch (error: any) {
      // Handle specific validation errors
      if (error.status === 401) {
        throw new AIProviderError(
          'Invalid OpenAI API key. Please verify your API key from https://platform.openai.com/api-keys',
          AIErrorType.AUTHENTICATION,
          error.status
        );
      }
      
      if (error.status === 403) {
        throw new AIProviderError(
          'Access forbidden. Your API key may not have permission for this model or your account may have insufficient credits.',
          AIErrorType.AUTHENTICATION,
          error.status
        );
      }
      
      if (error.status === 404 && error.message?.includes('model')) {
        throw new AIProviderError(
          `Model '${this.defaultModel}' is not available. Please check the model name or your API access level.`,
          AIErrorType.MODEL_NOT_FOUND,
          error.status
        );
      }
      
      // For other errors during validation, log but don't fail initialization
      // They might be temporary network issues
      console.warn('OpenAI connection validation warning:', error.message);
    }
  }

  /**
   * Validates model name format for OpenAI models.
   * 
   * @private
   * @param modelName - Model name to validate
   * @throws {AIProviderError} If model name format is invalid
   */
  private validateModelName(modelName: string): void {
    if (!modelName || typeof modelName !== 'string') {
      throw new AIProviderError(
        'Model name must be a non-empty string',
        AIErrorType.INVALID_REQUEST
      );
    }

    // OpenAI model names follow specific patterns
    const validPatterns = [
      /^gpt-4o(?:-mini)?(?:-\d{4}-\d{2}-\d{2})?$/,           // e.g., gpt-4o, gpt-4o-mini, gpt-4o-2024-11-20
      /^gpt-4(?:-turbo)?(?:-\d{4}-\d{2}-\d{2})?$/,          // e.g., gpt-4, gpt-4-turbo, gpt-4-turbo-2024-04-09
      /^gpt-3\.5-turbo(?:-\d{4})?$/,                        // e.g., gpt-3.5-turbo, gpt-3.5-turbo-0125
      /^gpt-4-\d{4}-preview$/,                              // e.g., gpt-4-0125-preview
      /^text-davinci-\d{3}$/,                               // Legacy models
      /^ft:.+$/                                             // Fine-tuned models
    ];

    const isValid = validPatterns.some(pattern => pattern.test(modelName));
    
    if (!isValid) {
      console.warn(`Model name '${modelName}' doesn't match expected OpenAI naming patterns. This may cause API errors.`);
    }
  }

  /**
   * Builds optimized request parameters for OpenAI API.
   * 
   * @private
   * @param params - Input completion parameters
   * @param stream - Whether to enable streaming
   * @returns Formatted request parameters
   */
  private buildRequestParams(params: CompletionParams, stream: boolean) {
    return {
      model: params.model || this.defaultModel,
      messages: this.convertMessages(params.messages),
      max_tokens: params.maxTokens || 1000,
      temperature: params.temperature ?? 0.7,
      top_p: params.topP,
      stop: params.stopSequences,
      stream,
      // Enable structured outputs for better reliability
      stream_options: stream ? { include_usage: true } : undefined
    };
  }

  /**
   * Converts generic messages to OpenAI's chat completion format.
   * 
   * OpenAI supports system messages natively within the conversation flow,
   * making this conversion straightforward.
   * 
   * @private
   * @param messages - Input messages array
   * @returns OpenAI-formatted messages
   */
  private convertMessages(messages: AIMessage[]): OpenAI.Chat.Completions.ChatCompletionMessageParam[] {
    return messages.map(message => ({
      role: message.role as 'system' | 'user' | 'assistant',
      content: message.content
    }));
  }

  /**
   * Processes streaming response chunks from OpenAI API.
   * 
   * @private
   * @param stream - OpenAI streaming response promise
   * @returns AsyncIterable of formatted completion chunks
   */
  private async *processStreamChunks(stream: Promise<any>): AsyncIterable<CompletionChunk> {
    const state: StreamingState = {
      messageId: '',
      promptTokens: 0,
      completionTokens: 0,
      started: false
    };

    try {
      const streamResponse = await stream;
      
      for await (const chunk of streamResponse) {
        // Track message ID
        if (chunk.id && !state.messageId) {
          state.messageId = chunk.id;
        }

        // Handle content chunks
        const choice = chunk.choices?.[0];
        if (choice?.delta?.content) {
          state.started = true;
          yield {
            content: choice.delta.content,
            isComplete: false,
            id: state.messageId || chunk.id
          };
        }

        // Handle completion with usage information
        if (choice?.finish_reason || chunk.usage) {
          // Extract usage information if available
          if (chunk.usage) {
            state.promptTokens = chunk.usage.prompt_tokens || 0;
            state.completionTokens = chunk.usage.completion_tokens || 0;
          }

          yield {
            content: '',
            isComplete: true,
            id: state.messageId || chunk.id,
            usage: {
              promptTokens: state.promptTokens,
              completionTokens: state.completionTokens,
              totalTokens: state.promptTokens + state.completionTokens
            }
          };
          return;
        }
      }
    } catch (error) {
      // Handle streaming errors gracefully
      throw new AIProviderError(
        `Streaming interrupted: ${(error as Error).message}`,
        AIErrorType.NETWORK,
        undefined,
        error as Error
      );
    }
  }

  /**
   * Formats OpenAI's completion response to our standard interface.
   * 
   * @private
   * @param response - Raw OpenAI API response
   * @returns Formatted completion response
   * @throws {AIProviderError} If response format is unexpected
   */
  private formatCompletionResponse(response: OpenAI.Chat.Completions.ChatCompletion): CompletionResponse {
    const choice = response.choices[0];
    if (!choice || !choice.message.content) {
      throw new AIProviderError(
        'No content found in OpenAI response',
        AIErrorType.UNKNOWN
      );
    }

    return {
      content: choice.message.content,
      model: response.model,
      usage: {
        promptTokens: response.usage?.prompt_tokens || 0,
        completionTokens: response.usage?.completion_tokens || 0,
        totalTokens: response.usage?.total_tokens || 0
      },
      id: response.id,
      metadata: {
        finishReason: choice.finish_reason,
        systemFingerprint: response.system_fingerprint,
        created: response.created
      }
    };
  }

  /**
   * Handles and transforms OpenAI-specific errors.
   * 
   * This method maps OpenAI's error responses to our standardized
   * error format, providing helpful context and actionable suggestions.
   * 
   * @private
   * @param error - Original error from OpenAI API
   * @returns Normalized AIProviderError
   */
  private handleOpenAIError(error: any): AIProviderError {
    if (error instanceof AIProviderError) {
      return error;
    }

    const message = error.message || 'Unknown OpenAI API error';
    const status = error.status || error.statusCode;

    // Map OpenAI-specific error codes and types
    switch (status) {
      case 400:
        if (message.includes('max_tokens')) {
          return new AIProviderError(
            'Max tokens value exceeds model limit. Please reduce max_tokens or use a model with higher limits.',
            AIErrorType.INVALID_REQUEST,
            status,
            error
          );
        }
        if (message.includes('model')) {
          return new AIProviderError(
            'Invalid model specified. Please check model name and availability.',
            AIErrorType.MODEL_NOT_FOUND,
            status,
            error
          );
        }
        if (message.includes('messages')) {
          return new AIProviderError(
            'Invalid message format. Please check message structure and content.',
            AIErrorType.INVALID_REQUEST,
            status,
            error
          );
        }
        return new AIProviderError(
          `Invalid request: ${message}`,
          AIErrorType.INVALID_REQUEST,
          status,
          error
        );

      case 401:
        return new AIProviderError(
          'Authentication failed. Please verify your OpenAI API key from https://platform.openai.com/api-keys',
          AIErrorType.AUTHENTICATION,
          status,
          error
        );

      case 403:
        if (message.includes('billing') || message.includes('quota')) {
          return new AIProviderError(
            'Insufficient quota or billing issue. Please check your OpenAI account billing and usage limits.',
            AIErrorType.AUTHENTICATION,
            status,
            error
          );
        }
        return new AIProviderError(
          'Access forbidden. Your API key may not have permission for this model or feature.',
          AIErrorType.AUTHENTICATION,
          status,
          error
        );

      case 404:
        return new AIProviderError(
          'Model not found. The specified OpenAI model may not exist or be available to your account.',
          AIErrorType.MODEL_NOT_FOUND,
          status,
          error
        );

      case 429:
        if (message.includes('rate')) {
          return new AIProviderError(
            'Rate limit exceeded. Please reduce request frequency and implement exponential backoff.',
            AIErrorType.RATE_LIMIT,
            status,
            error
          );
        }
        if (message.includes('quota')) {
          return new AIProviderError(
            'Usage quota exceeded. Please check your OpenAI usage limits and billing.',
            AIErrorType.RATE_LIMIT,
            status,
            error
          );
        }
        return new AIProviderError(
          'Too many requests. Please slow down and try again.',
          AIErrorType.RATE_LIMIT,
          status,
          error
        );

      case 500:
      case 502:
      case 503:
      case 504:
        return new AIProviderError(
          'OpenAI service temporarily unavailable. Please try again in a few moments.',
          AIErrorType.NETWORK,
          status,
          error
        );

      default:
        // Handle timeout and network errors
        if (message.includes('timeout') || error.code === 'ETIMEDOUT') {
          return new AIProviderError(
            'Request timed out. OpenAI may be experiencing high load.',
            AIErrorType.TIMEOUT,
            status,
            error
          );
        }

        if (message.includes('network') || error.code === 'ECONNREFUSED') {
          return new AIProviderError(
            'Network error connecting to OpenAI servers.',
            AIErrorType.NETWORK,
            status,
            error
          );
        }

        return new AIProviderError(
          `OpenAI API error: ${message}`,
          AIErrorType.UNKNOWN,
          status,
          error
        );
    }
  }
} 